#!/bin/bash
set -e

usage() {
    echo; echo "Usage: $0 --fasta=/Path/to/infile.fasta --database=/Path/to/RAPSEARCH_DB --out=/Path/to/output.out [--evalue=1e-5] [--threads=4]"
    echo "  --fasta     Path to the input AA FASTA file"
    echo "  --out       Path to the output TXT file"
    echo "  --database  Path to the RapSearch2 database index base"
    echo "  --evalue    E value cut off. (Defualt = 1e-5)"
    echo "  --threads   Number of threads the pipeline use (Default=1)"
    echo "  -h, --help  Print this help message out"; echo;
    exit 1;
}

## ========================
## Globals
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
PIPELINES="${DIR}"
SCRIPTS="${PIPELINES}../scripts"
THREADS=1
EVALUE="1e-5"

## ========================
## Process the input parameters
if [ $# -gt 5 ] || [ $# -lt 3 ]
then
    usage
fi

while true
do
    case $1 in
    --help|-h)
	    usage
	    exit;;
    --fasta=?*)
	    FASTA=${1#*=};;
    --fasta|fasta=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --out=?*)
	    OUT=${1#*=};;
    --out|out=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --evalue=?*)
	    EVALUE=${1#*=};;
    --evalue|evalue=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --threads=?*)
	    THREADS=${1#*=};;
    --threads|threads=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --database=?*)
            DATABASE=${1#*=};;
    --database|database=)
            echo "$0: missing argument for '$1' option"
            usage
            exit 1;;
    --)
	    shift
	    break;;
    -?*)
	    echo "$0: invalid option: $1"
	    usage
	    exit 1;;
    *)
	    break
    esac
    shift
done

##============
## Logging
LOG="$( cd "$( dirname "${OUT}" )" && pwd )"
LOGTIME="${LOG}/rapsearch2.log"

## Run RAPsearch
command time -v -o "${LOGTIME}" -a rapsearch -q "${FASTA}" \
    -d "${DATABASE}" \
    -o "${OUT}" \
    -z "${THREADS}" \
    -e "${EVALUE}" \
    -t a > "${LOGTIME}" 2>&1
status=$?

exit $status
